/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// IV-12 VFD Four Digit Clock Project - 2024.11 - tomcii
// Changelog:
// 2024.11.04 - Programming initiated
// 2024.11.04 - Pin declaration // ATMEGA32-16PU
// 2024.11.07 - Daylight saving time for GMT+1 added - *** WORKS PROPERLY ***
// 2024.11.08 - Display correction
//            - Implement Pushbutton1 to PB6 (D6) input - Done
//            - Implement button debounce to the PB1 - Done
//            - Implement Celsius reading from DS3231 - Done
//            - Implement byte degreeSymbol1 and byte degreeSymbol2 and attach them to the numtables
//            - Implement switch-case structure for the display 
//                  case 1: Displaying the time
//                  case 2: Displaying the year - month - day by two sequence (First will be the year - then wait 2 seconds and then month and day.
//                  case 3: Displaying the temperature
// 2024.11.18 - Fist upload to ATMEGA32-16U -  MightyCore does the trick. With USBasp it was successfull by USBasp slow first, and then did the Bootloader
//            (which updated the fuses) and from now on it is working with USBasp
//            - Program uploaded (wiring is also okay for USBasp) 
// 2025.01.24 - Program separated to tabs to a better overview and modification
/***************************************************************************************************************************************************************/
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
#include <Wire.h>
#include <DS3231.h>
#define DS3231 B1101000               // I2C address
const int pinPB = 6;                  // Pushbutton 1
int buttonState;                      // the current reading from the input pin
int lastButtonState = 0;            // the previous reading from the input pin 
unsigned long lastDebounceTime = 0;  // the last time the output pin was toggled
const unsigned long debounceDelay = 100;     // the debounce time; increase if the output flickers
int readingButton = 0;
int switchCase = 0;
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
// IV-12 VFD Pinout
//  a = 8
//  b = 7
//  c = 6
//  d = 5
//  e = 1
//  f = 10
//  g = 9
//       a
//       _
//   f /_ / b
//  e /_ / c
//     d
// Middle is G (there is no decimal point on IV-12 VFD)
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////                    
byte segment_digit1[] = {0, 1, 2, 3, 4, 5, 8};                      
byte segment_digit2[] = {9, 10, 11, 12, 13, 14, 15};  
byte segment_digit3[] = {18, 19, 20, 21, 22, 23, 31};               
byte segment_digit4[] = {30, 29, 28, 27, 26, 25, 24};    
/***************************************************************************************************************************************************************/                            
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////                             
byte second=0, minute=0, hour=0, weekday=0, day=0, month=0, year=0;
byte year_1000 = 2;
byte year_100 = 0;
byte DECTOBCD(byte val);
byte BCDTODEC(byte val);
byte    Temperature = 0;            // Temperature variable
int8_t  MostSignificantBit = 0;     // MSB will be the whole part of the temperature LSB - Fraction of the temperature - Not used
byte    Calibration = 6;            // Calibration flag
byte degree = 10;                   // To display degree symbol and C 

void updateTime();
void printTime();
void print7Digit(byte number);

bool isDaylightSavingTime(int year, int month, int day, int weekday) {
  // The Daylight saving time will starts at the last sunday of March
  if (month == 3 && (day + 7 - weekday) > 31) {
    return true;
  }
  // The daylight saving time will ends at the last sunday of October
  if (month == 10 && (day + 7 - weekday) > 31) {
    return false;
  }
  // From April to September it is always daylight saving time
  if (month > 3 && month < 10) {
    return true;
  }
  //The default is the winter time
  return false;
}
/***************************************************************************************************************************************************************/
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void setup() {
  byte init_segm;                 // Byte for the segments initialization (for for cycle)
  Wire.begin();                   // Begin I2C communication 
  // Initialize segments and pull them down
  for (init_segm=0; init_segm<7; init_segm++) {
    pinMode(segment_digit1[init_segm], OUTPUT);
    pinMode(segment_digit2[init_segm], OUTPUT);
    pinMode(segment_digit3[init_segm], OUTPUT);
    pinMode(segment_digit4[init_segm], OUTPUT);
    pinMode(pinPB, INPUT);
    digitalWrite(segment_digit1[init_segm], HIGH);
    digitalWrite(segment_digit2[init_segm], HIGH);
    digitalWrite(segment_digit3[init_segm], HIGH);
    digitalWrite(segment_digit4[init_segm], HIGH);
  }
}
/***************************************************************************************************************************************************************/
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void loop() {
  updateSubroutine();               // Call updateTime subroutine - Updates and reads all the things which is needed for this project from the DS3231 each cycle
  printSubroutine();                // Call printTime subroutine
  checkButton();                    // Call checkButton subroutine
}
